unit CaptureUtils;

interface

uses Windows, DirectShow;

function FilterName(filter: IBaseFilter): string;
function PinName(pin: IPin): string;

function DumpGraph(Graph: IGraphBuilder): string;

implementation

uses StrUtils;

const
  Separator = #$0d#$0A;
  Separator1 = #$0d#$0A'--------------------------------------'#$0d#$0A+
               ' Filter : ';


function FilterName(filter: IBaseFilter): string;
var
  fi: TFilterInfo;
begin
  if (filter<>nil) and Succeeded(filter.QueryFilterInfo(fi)) then begin
    fi.pGraph:= nil;
    Result:= fi.achName;
  end;
end;

function PinName(pin: IPin): string;
var
  pi: TPin_Info;
begin
  if (pin=nil) then Exit;
  if Succeeded(pin.QueryPinInfo(pi)) then begin
    Result:= FilterName(pi.pFilter)+'.'+pi.achName;
    pi.pFilter:= nil;
  end;
end;

function DumpGraph(Graph: IGraphBuilder): string;
var
  filters: IEnumFilters;
  pin, pin1: IPin;
  pins: IEnumPins;
  filter: IBaseFilter;
  pd: TPin_Direction;
begin
  if (Graph=nil) or Failed(Graph.EnumFilters(filters)) then Exit;
  if Succeeded(filters.Reset) then
    while true do begin
      if Failed(filters.Next(1, filter, nil)) or (filter=nil) then break;
      // filter info
      Result:= Result+Separator+Separator1;
      Result:= Result+FilterName(filter)+Separator;

      if Failed(filter.EnumPins(pins)) or Failed(pins.Reset) then continue;
        while true do begin
          if Failed(pins.Next(1, pin, nil)) or (pin=nil) then break;

          // pin info
          Result:= Result+Separator;
          Result:= Result + AddCharR(' ', PinName(pin), 25);

          pin.QueryDirection(pd);
          if pd=PINDIR_INPUT then Result:= Result+'<--'#9
                             else Result:= Result+'-->'#9;
          if Succeeded(pin.ConnectedTo(pin1)) then begin
            Result:= Result + PinName(pin1);
            pin1:= nil;
          end;

        end;
    end;
end;


end.
