unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DirectShow, Menus, VCap, StdCtrls, ExtCtrls, Placemnt, ComCtrls, Buttons,
  Mask, ToolEdit;

type
  TFormMain = class(TForm)
    StatusBar: TStatusBar;
    Panel2: TPanel;
    Panel3: TPanel;
    cap: TVideoCapture;
    Image: TImage;
    FormStorage: TFormStorage;
    Panel6: TPanel;
    btStart: TBitBtn;
    btStop: TBitBtn;
    Panel4: TPanel;
    Label7: TLabel;
    lbDialogs: TListBox;
    Panel5: TPanel;
    Timer: TTimer;
    Panel1: TPanel;
    Button2: TButton;
    Button1: TButton;
    Panel7: TPanel;
    Label3: TLabel;
    cbVModes: TComboBox;
    Label4: TLabel;
    cbAModes: TComboBox;
    Bevel1: TBevel;
    Label5: TLabel;
    feBitmap: TFilenameEdit;
    btCaptureFrame: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure capCaptureProgress(Sender: TObject);
    procedure btStartClick(Sender: TObject);
    procedure btStopClick(Sender: TObject);
    procedure btCaptureFrameClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure capChangeDevice(Sender: TObject);
    procedure cbVModesChange(Sender: TObject);
    procedure cbAModesChange(Sender: TObject);
    procedure lbDialogsDblClick(Sender: TObject);
    procedure capStartPreview(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormStorageRestorePlacement(Sender: TObject);
    procedure FormStorageSavePlacement(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure capBitmapGrabbed(CapturedImage: TCapturedBitmap);
  private
    config: TGraphConfig;
    function InitCapture : Boolean;
  public
  end;

var
  FormMain: TFormMain;

implementation

uses VCLUtils, CaptureUtils, ActiveX, GraphConfig;

{$R *.DFM}

function TFormMain.InitCapture: Boolean;
begin
  Result:= cap.Init;
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  if not InitCapture then begin
    ShowMessage('Can''t init capture!');
    Application.Terminate;
  end;
end;

procedure TFormMain.capCaptureProgress(Sender: TObject);
begin
  StatusBar.SimpleText:= 'Frames Captured '+IntToStr(cap.NotDropped)+
                         '. Frames Dropped '+IntToStr(cap.FramesDropped)+
                         '. Capture time '+IntToStr(cap.CapTime div 1000)+'.'+IntToStr((cap.CapTime div 10) mod 100);
  btStart.Enabled:= not cap.Capturing;
  btStop.Enabled:= not btStart.Enabled;
end;

procedure TFormMain.btStartClick(Sender: TObject);
begin
  cap.StartCapture(true);
  capCaptureProgress(Self);
end;

procedure TFormMain.btStopClick(Sender: TObject);
begin
  cap.StopCapture;
  capCaptureProgress(Self);
end;

procedure TFormMain.btCaptureFrameClick(Sender: TObject);
begin
  if not cap.CaptureFrame then ShowMessage('Can''t capture frame!');
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  cap.StopPreview;
  cap.StopCapture;
end;

procedure TFormMain.capChangeDevice(Sender: TObject);
var
  i, capCount: integer;
  cvcapMode, vcapMode: TVCapMode;
  cacapMode, acapMode: TACapMode;
  s: string;
  d: TCaptureDialog;
begin
  cvcapMode:= cap.VCapMode;
  cacapMode:= cap.ACapMode;

  cbVModes.Items.Clear;
  capCount:= cap.VCapModeCount;
  for i:= 0 to capCount-1 do begin
    vcapMode:= cap.VCapModes[i];
    s:= GetModeString(vcapMode);
    cbVModes.Items.Add(s);
    if IsEqualModes(cvcapMode, vcapMode) then
      cbVModes.ItemIndex:= i;
  end;

  cbAModes.Items.Clear;
  capCount:= cap.ACapModeCount;
  for i:= 0 to capCount-1 do begin
    acapMode:= cap.ACapModes[i];
    s:= GetModeString(acapMode);
    cbAModes.Items.Add(s);
    if IsEqualModes(cacapMode, acapMode) then
      cbAModes.ItemIndex:= i;
  end;

  lbDialogs.Items.Clear;
  for d:= Low(TCaptureDialog) to High(TCaptureDialog) do
    if d in cap.Dialogs then begin
      lbDialogs.Items.AddObject(DialogTitles[d], TObject(d));
    end;
end;

procedure TFormMain.capStartPreview(Sender: TObject);
begin
// refresh cur modes
  cbVModes.ItemIndex:= cap.VCapModeIdx;
  cbAModes.ItemIndex:= cap.VCapModeIdx;
end;

procedure TFormMain.cbVModesChange(Sender: TObject);
begin
  cap.SetVCapMode(cbVModes.ItemIndex);
  cap.StartPreview;
end;

procedure TFormMain.cbAModesChange(Sender: TObject);
begin
  cap.SetACapMode(cbAModes.ItemIndex);
  cap.StartPreview;
end;

procedure TFormMain.lbDialogsDblClick(Sender: TObject);
begin
  if (lbDialogs.Items.Count>0) and (lbDialogs.ItemIndex>=0) then
    cap.ShowDialog(TCaptureDialog(lbDialogs.Items.Objects[lbDialogs.ItemIndex]));
end;

procedure TFormMain.TimerTimer(Sender: TObject);
begin
  // make a delay to show form (let form get a handle)
  Timer.Enabled:= false;
  StartWait;
  try
    cap.RestoreGraph(config);
  finally
    StopWait;
  end;
end;

procedure TFormMain.Button1Click(Sender: TObject);
var
  s: TStringList;
  x: IXml2Dex;
begin
  if (cap.CaptureGraph=nil) then Exit;
  s:= TStringList.Create;
  try
    s.Text:= DumpGraph(cap.CaptureGraph);
    s.SaveToFile('graph.txt');
  finally
    s.Free;
  end;
  if Succeeded(CoCreateInstance(CLSID_Xml2Dex, nil, CLSCTX_INPROC, IID_IXml2Dex, x)) then
    x.WriteGrfFile(cap.CaptureGraph, 'graph.grf');
end;

procedure TFormMain.FormStorageRestorePlacement(Sender: TObject);
var s: string;
begin
  s:= FormStorage.IniFile.ReadString('Graph', 'Config', '');
  config.RestoreGraph(s);
  Timer.Enabled:= s<>'';
end;

procedure TFormMain.FormStorageSavePlacement(Sender: TObject);
begin
  cap.SaveGraph(config);
  FormStorage.IniFile.WriteString('Graph', 'Config', config.SaveGraph);
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  config:= TGraphConfig.Create;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  config.Free;
end;

procedure TFormMain.Button2Click(Sender: TObject);
begin
  cap.SaveGraph(config);
  if EditGraphConfig(config) then begin
    StartWait;
    try
      cap.RestoreGraph(config);
    finally
      StopWait;
    end;
  end;
end;

procedure TFormMain.capBitmapGrabbed(CapturedImage: TCapturedBitmap);
begin
  CapturedImage.SaveToFile(feBitmap.FileName);
  Image.Picture.Assign(CapturedImage);
end;

end.
